︠f0407ff8-43f4-486c-8fdd-2edd46ed87ads︠
# Kaip papildyti vektorių sistemą iki erdvės bazės?

def papildom_poerdviai(W, V):
    """
    Gražina tiesinės erdvės V poerdvio W papildinio iki V bazę.
    Kitaip sakant, randa tokį poerdvį U, kad V yra W ir U tiesioginė
    suma ir gražina jo bazę.
    """
    Q, pi, lift = V.quotient_abstract(W)
    Basis = [lift(v) for v in Q.basis()]
    return Basis

# pi   yra projekcijos atvaizdis iš erdvės V į faktorerdvę V/W: pi(v) = v + W.
# lift yra "atvirkštinis" atvaizdis iš faktorerdvės Q = V/W  į  erdvę V, tenkinantis
# lygybę pi(lift(u)) = u  su visais u iš Q.


def papildom(vec, V):
    """
    Tiesinės erdvės V vektorių šeimą vec papildo vektoriais v1,..., vn iki
    erdvės V bazės ir gražina papildytus vektorius v1,..., vn.
    """
    return papildom_poerdviai(V.span(vec), V)

︡6bff092b-a8cc-487d-a360-3af02fb2be58︡{"done":true}
︠eb616ba1-fa26-41ef-a4ca-ca91e8784179s︠
# 2 pavyzdys. Rasime matricos

A = matrix(QQ, 4, 4, [[1, 1, -2, 0],
                      [2, 1, 0, 2],
                      [1, 0, 1, 1],
                      [0,-1,2,1]])
show(A)
︡b253a477-600e-4619-9ed0-41807b0f4138︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rrrr}\n1 &amp; 1 &amp; -2 &amp; 0 \\\\\n2 &amp; 1 &amp; 0 &amp; 2 \\\\\n1 &amp; 0 &amp; 1 &amp; 1 \\\\\n0 &amp; -1 &amp; 2 &amp; 1\n\\end{array}\\right)$</div>"}︡{"done":true}
︠ab4244ce-99b2-46fd-80b5-644d1ecb786c︠
# Žodano formą ir atitinkamą bazę.
︡affd63e5-63cc-4149-9317-875eac9ae867︡
︠93cdbdc9-9613-4440-8f13-834b69af7c3es︠
# Randame matricos charakteristinį polinomą:

p = A.charpoly(var = 't')
show(p)
show(p.factor())
︡d2760db8-3e98-4424-b8eb-09e2ee50fac6︡{"html":"<div align='center'>$\\displaystyle t^{4} - 4 t^{3} + 6 t^{2} - 4 t + 1$</div>"}︡{"html":"<div align='center'>$\\displaystyle (t - 1)^{4}$</div>"}︡{"done":true}
︠026db0e1-9997-42f6-ad96-bc14a86b8d66s︠
# Nagrinėjame matricą B := A-1*E. Ieškome mažiausoi p: rg(B^p)=4-4=0:
B = A - 1*matrix.identity(4)
show(B)
︡712263b5-beb4-4a0e-8675-744698c8dd5f︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rrrr}\n0 &amp; 1 &amp; -2 &amp; 0 \\\\\n2 &amp; 0 &amp; 0 &amp; 2 \\\\\n1 &amp; 0 &amp; 0 &amp; 1 \\\\\n0 &amp; -1 &amp; 2 &amp; 0\n\\end{array}\\right)$</div>"}︡{"done":true}
︠91e33067-49ed-46c0-a6d3-eb8c26e783b3s︠
# Matricos B laipsnių rangai:
B.rank()
(B**2).rank()
︡8f6e7eef-198c-4bba-a3a9-d78d2d55d9f6︡{"stdout":"2\n"}︡{"stdout":"0\n"}︡{"done":true}
︠561a0d81-5b90-4a8e-a9d8-548373343eads︠
# Taigi, p=2. Matricos B branduolys ker B ir jo dimensija:
L1 = B.left_kernel()
L1.dimension()
︡482bd9fa-f98a-4421-9545-949932e9ddf1︡{"stdout":"2\n"}︡{"done":true}
︠a6c1524a-addb-4e13-bc9a-005565cc8c1ds︠
# Matricos B^2 branduolys ker B^2 ir jo dimensija:
L2 = (B**2).left_kernel()
L2.dimension()
︡db39a400-e3e3-409b-9af3-b9a59235e78e︡{"stdout":"4\n"}︡{"done":true}
︠05092db0-b874-4fdd-bc64-ff1eeb567ec8s︠
# Poerdvio L1 bazę pildant iki erdvės L2 bazės, reikia pridėti 2 vektorius v11 ir v12:
L = papildom(L1.basis(), L2)
v11 = L[0]
v12 = L[1]
v11
v12
︡517a5e78-7216-48cd-9747-21be8edf64e1︡{"stdout":"(1, 0, 0, 0)\n"}︡{"stdout":"(0, 1, 0, 0)\n"}︡{"done":true}
︠74376d36-bce7-4bdd-990f-9e170a809825s︠
# Suskaičiuojame vektorius v11B ir v12B:
v11*B
v12*B
︡1cfc13cf-3ea8-44fe-80cf-6666dcd98585︡{"stdout":"(0, 1, -2, 0)\n"}︡{"stdout":"(2, 0, 0, 2)\n"}︡{"done":true}
︠171492ee-30e9-4a28-948e-6b95af7b2f2bs︠
# Taigi radome Žordano bazę v11,v11B,v21,v22. 
# Perėjimo matrica (iš standartinės bazės į Žordano bazę):

T = matrix(QQ,4,4,[v11,v11*B,v12,v12*B])
show(T)
︡c9415b16-237c-4f37-9abf-889a46646f93︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rrrr}\n1 &amp; 0 &amp; 0 &amp; 0 \\\\\n0 &amp; 1 &amp; -2 &amp; 0 \\\\\n0 &amp; 1 &amp; 0 &amp; 0 \\\\\n2 &amp; 0 &amp; 0 &amp; 2\n\\end{array}\\right)$</div>"}︡{"done":true}
︠37f72aa0-d152-4557-b9a3-8bccf955e21cs︠
# Pradinės matricos Žordano matrica:

show(T*A*(T^(-1)))
︡5b647669-8075-48b2-9550-cbf721aeba73︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{rrrr}\n1 &amp; 1 &amp; 0 &amp; 0 \\\\\n0 &amp; 1 &amp; 0 &amp; 0 \\\\\n0 &amp; 0 &amp; 1 &amp; 1 \\\\\n0 &amp; 0 &amp; 0 &amp; 1\n\\end{array}\\right)$</div>"}︡{"done":true}
︠ee05924e-c11d-48e6-a106-c0569d3049a0︠









